-- Example of a binary synchronous counter 
--  using FSM style

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY Freq_Div_50 IS
      Port (	CLK 	: IN	STD_LOGIC;
		CD 				: IN	STD_LOGIC;
		CE 				: IN	STD_LOGIC;
		-- Q 				: OUT	STD_LOGIC_VECTOR(23 DOWNTO 0);
		TC50		: OUT	STD_LOGIC 
		
	);

END Freq_Div_50;

			-- Internal desciption in FSM style

ARCHITECTURE FSM_like OF Freq_Div_50 IS
-- This is the real and necessary Max_Count:
-- CONSTANT Max_Count	: STD_LOGIC_VECTOR(23 DOWNTO 0) := "110000000001000111101011"; -- terminal_count after 10 states 12587500

-- This the the Max_Count modified to speed simulations. 
-- So you must comment this line and activate the above one if you plan to use the real UP2 board
CONSTANT Max_Count	: STD_LOGIC_VECTOR(5 DOWNTO 0) := "110001"; 


CONSTANT Reset		: STD_LOGIC_VECTOR(5 DOWNTO 0) := "000000"; -- 1100 0000 0001 0001 1110 1100 --1100 0000 0001 0001 1110 1011

-- Internal wires 

SIGNAL present_state,future_state: STD_LOGIC_VECTOR(5 downto 0) := Reset;

BEGIN
------------------------------ the only clocked block : the state register
state_register: PROCESS (CD, CLK)
	BEGIN
						
	IF CD = '1' THEN 			-- reset counter
		present_state <= Reset; 
	ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous register (D-type flip-flop)
		present_state <= future_state; 
	END IF;

END PROCESS state_register;

------------------------- ESS state_registercombinational system for calculating next state
CS_1: PROCESS (present_state, CE)
	BEGIN
	IF CE = '1' THEN
		IF(present_state < Max_Count ) THEN
			future_state <= present_state + 1 ;
		ELSE
			future_state <= Reset;
		END IF;
	ELSE 	
		future_state <= present_state;  -- count disable
	END IF;

END PROCESS CS_1;

------------------------- CS_2: combinational system for calculating extra outputs 
-------------------------- and outputing the present state (the actual count)

TC50 <= '1' WHEN ((present_state = Max_Count)AND CE = '1') ELSE '0';  --terminal count
-- Q <= present_state; In this application, there is no need to output the count value Q


END FSM_like;






