-- Transmitter unit for the UART module
LIBRARY ieee ;
USE ieee.std_logic_1164.all ;

ENTITY Transmitter_unit IS  

PORT (
		CLK 		: 	IN 	STD_LOGIC;
		CD			: 	IN 	STD_LOGIC;
		TX_IN		: 	IN 	STD_LOGIC_VECTOR (7 DOWNTO 0);		
		ST			: 	IN 	STD_LOGIC;
		Enable_TX 	: 	IN 	STD_LOGIC;
		ET			:  	OUT	STD_LOGIC;
		Serial_out	: 	OUT STD_LOGIC
		);
END Transmitter_unit; 


ARCHITECTURE structural OF Transmitter_unit IS
    

COMPONENT  transmitter_datapath IS  	
	PORT (
		CLK			:	IN STD_LOGIC;
		LDD			: 	IN STD_LOGIC;
		CE			: 	IN STD_LOGIC;
		CD			:	IN STD_LOGIC;
		TX_IN		: 	IN STD_LOGIC_VECTOR (7 DOWNTO 0);
		S			:	IN STD_LOGIC_VECTOR (1 DOWNTO 0);		
		F			:	IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		TC8			:	OUT STD_LOGIC;
		Serial_out	: 	OUT STD_LOGIC
		);
	END COMPONENT;
	
COMPONENT transmitter_control_unit IS
	PORT(
			CLK, CD, Enable_TX, ST	:	IN  STD_LOGIC;	
			TC8						:	IN  STD_LOGIC;
		    F               		:   OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
			S               		:   OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
			LDD						:	OUT std_logic;
			ET						:	OUT std_logic;
			CE  					:	OUT std_logic			
         );
	END COMPONENT;

   SIGNAL  F,S  		: STD_LOGIC_VECTOR (1 DOWNTO 0);
   SIGNAL  TC8, LDD, CE	: STD_LOGIC;     
   
BEGIN
	
Chip1	:	transmitter_control_unit
   	    PORT MAP (
	   		CLK       => CLK,
			CD        => CD,
			Enable_TX => Enable_TX,
			ST        => ST,
            TC8       => TC8,
            F         => F,
            S         => S,
            CE        => CE,
            LDD       => LDD,
			ET        => ET
			    );
		   
Chip2	: 	transmitter_datapath
  	    PORT MAP (
			CLK       	=> CLK,
			LDD         => LDD,
			CE          => CE,
			CD          => CD,
			TX_IN       => TX_IN,
			S           => S,	
			F           => F,
			TC8         => TC8,
 			Serial_out	=> Serial_out					  
		   );
		   

END structural ;